package shape
{
	import flash.display.Sprite;
	import geom.Triangle;
	import geom.Vector3D;
	import geom.Matrix4;
	import flash.geom.Point;
	
	public class Box extends Sprite
	{
		private var faces:Array = new Array();
		private var ambCol:Vector3D = new Vector3D(0,0,0);	// 環境光
		private var difCol:Vector3D = new Vector3D(0,0,0);	// 拡散反射光
		private var pos:Point = new Point();
		
		public var rotMat:Matrix4 = new Matrix4();
		
		public function Box(size:uint)
		{
			var s:Number = size / 2.0;
			
			// 頂点
			var v0:Vector3D = new Vector3D(-s,s,s);
			var v1:Vector3D = new Vector3D(-s,s,-s);
			var v2:Vector3D = new Vector3D(s,s,s);
			var v3:Vector3D = new Vector3D(s,s,-s);
			var v4:Vector3D = new Vector3D(-s,-s,-s);
			var v5:Vector3D = new Vector3D(s,-s,-s);
			var v6:Vector3D = new Vector3D(s,-s,s);
			var v7:Vector3D = new Vector3D(-s,-s,s);
			
			// 三角形
			faces[0] = new Triangle(v0,v1,v2);
			faces[1] = new Triangle(v2,v1,v3);
			faces[2] = new Triangle(v1,v4,v3);
			faces[3] = new Triangle(v3,v4,v5);
			faces[4] = new Triangle(v0,v7,v4);
			faces[5] = new Triangle(v0,v4,v1);
			faces[6] = new Triangle(v3,v5,v2);
			faces[7] = new Triangle(v2,v5,v6);
			faces[8] = new Triangle(v4,v6,v5);
			faces[9] = new Triangle(v4,v7,v6);
			faces[10] = new Triangle(v7,v0,v6);
			faces[11] = new Triangle(v0,v2,v6);
			
			// 単位行列にしとく
			rotMat.setIdentity();
		}

		public function draw(light:Vector3D):void
		{
			var color:Vector3D = new Vector3D(0,0,0);
			graphics.clear();
			// 各facesを塗りつぶす（法線ベクトルのｚ値が正のもののみ）
			for(var i:uint = 0; i < 12; i++)
			{
				var face:Triangle = ((Triangle)(faces[i])).transform(rotMat);
				if(face.normal.z > 0)
				{
					var w:Number = face.normal.dotProduct(light);
					if(w < 0)
						w = 0;
					color.x = (int)((ambCol.x + w*difCol.x) * 255);
					color.y = (int)((ambCol.y + w*difCol.y) * 255);
					color.z = (int)((ambCol.z + w*difCol.z) * 255);
					
					// サチる
					if(color.x < 0) color.x = 0; if(color.x > 255) color.x = 255;
					if(color.y < 0) color.y = 0; if(color.y > 255) color.y = 255;
					if(color.z < 0) color.z = 0; if(color.z > 255) color.z = 255;
					
					var col:int = (color.x << 16) | (color.y << 8) | color.z;
					var v1:Vector3D = face.vertex[0];
					var v2:Vector3D = face.vertex[1];
					var v3:Vector3D = face.vertex[2];
					
					graphics.moveTo(pos.x+v1.x,pos.y+v1.y);
					graphics.beginFill(col);
					graphics.lineTo(pos.x+v2.x,pos.y+v2.y);
					graphics.lineTo(pos.x+v3.x,pos.y+v3.y);
					graphics.lineTo(pos.x+v1.x,pos.y+v1.y);
					graphics.endFill();
				}
			}
		}
		
		public function setAmbientColor(r:Number, g:Number, b:Number):void
		{
			ambCol.x = r;
			ambCol.y = g;
			ambCol.z = b;
		}
		
		public function setDiffuseColor(r:Number, g:Number, b:Number):void
		{
			difCol.x = r;
			difCol.y = g;
			difCol.z = b;
		}
		
		// 箱の中心位置
		public function setPosition(x:int, y:int):void
		{
			pos.x = x;
			pos.y = y;
		}
	}
}